@@ -55,7 +55,7 @@ use Carp;
 use JSON::Parse '0.60_01', 'valid_json';
 use JSON::Create '0.30_04';
 use IO::Socket;
-our $VERSION = '0.00_08';
+our $VERSION = '0.00_09';
 
 sub new
 {
@@ -136,7 +136,7 @@ sub make_sock
 sub get
 {
     my ($sock, $json_msg) = @_;
-    $sock->send ($json_msg);
+    $sock->send ($json_msg . chr (0));
     my $got = '';
     my ($ok) = eval {
 	my $data;
@@ -145,8 +145,10 @@ sub get
 	    $data = '';
 	    $sock->recv ($data, $max);
 	    $got .= $data;
+	    if ($got =~ s/\x00//) {
+		last;
+	    }
 	}
-	$sock->close ();
 	1;
     };
     return ($got, $ok);
