@@ -3,11 +3,12 @@ use warnings;
 use strict;
 use Carp;
 use utf8;
-our $VERSION = '0.00_08';
+our $VERSION = '0.00_09';
 
 use IO::Socket;
+use IO::Select;
 use JSON::Create '0.30_04', ':all';
-use JSON::Parse '0.60_01', ':all';
+use JSON::Parse '0.61', ':all';
 
 $SIG{PIPE} = sub {
     croak "Aborting on SIGPIPE";
@@ -73,70 +74,97 @@ sub serve
 	Listen => 5,
 	LocalPort => $gs->{port},
     );
-    $gs->{server} = IO::Socket->new (%so);
-    if (! $gs->{server}) {
-	carp "Can't open socket: $@";
-    }
     if ($gs->{verbose}) {
 	vmsg ("Serving on $gs->{port}");
     }
-    while (1) {
-	my $got = '';
-	my ($ok) = eval {
-	    $gs->{client} = $gs->{server}->accept ();
-	    if ($gs->{verbose}) {
-		vmsg ("Got a message");
-	    }
-	    my $data;
-	    my $max = 1000;
-	    # It might be better to use the atmark method on $gs->{client}
-	    # here.
-	    while (! defined $data || length ($data) == $max) {
-		$data = '';
-		$gs->{client}->recv ($data, $max);
-		$got .= $data;
-	    }
-	    1;
-	};
-	if (! $ok) {
-	    carp "accept failed: $@";
-	    next;
-	}
+    my $server = IO::Socket->new (%so);
+    if (! $server) {
+	carp "Error from IO::Socket->new: $@";
+	return;
+    }
+    my $s = IO::Select->new ();
+    $s->add ($server);
+    while (my @ready = $s->can_read ()) {
 	if ($gs->{verbose}) {
-	    vmsg ("Received " . length ($got) . " bytes of data");
+	    vmsg ("Reading from @ready");
 	}
-	if (! valid_json ($got)) {
+	for my $fh (@ready) {
+	    if ($fh == $server) {
+		my $new = $server->accept ();
+		$s->add ($new);
+		next;
+	    }
+	    my $got = '';
+	    my ($ok) = eval {
+		if ($gs->{verbose}) {
+		    vmsg ("Got a message");
+		}
+		my $data;
+		my $max = 1000;
+		while (! defined $data || length ($data) == $max) {
+		    $data = '';
+		    $fh->recv ($data, $max);
+		    $got .= $data;
+		    if ($got =~ s/\x{00}$//) {
+			last;
+		    }
+		}
+		1;
+	    };
+	    if (! $ok) {
+		carp "accept failed: $@";
+		next;
+	    }
 	    if ($gs->{verbose}) {
-		vmsg ("Not valid json");
+		vmsg ("Received " . length ($got) . " bytes of data");
 	    }
-	    $gs->reply ({error => 'invalid JSON'});
-	    next;
-	}
-	if ($gs->{verbose}) {
-	    vmsg ("Validated as JSON");
-	}
-	my $input = $gs->{jp}->parse ($got);
-	my $control = $input->{'JSON::Server::control'};
-	if (defined $control) {
-	    if ($control eq 'stop') {
-		if ($gs->{verbose}) {
-		    vmsg ("Received control message to stop");
-		}
-		$gs->reply ({'JSON::Server::response' => 'stopping'});
-		if ($gs->{verbose}) {
-		    vmsg ("Responded to control message to stop");
-		}
-		return;
+	    if (length ($got) == 0) {
+		next;
 	    }
-	    warn "Unknown control command '$control'";
+	    if (! valid_json ($got)) {
+		if ($gs->{verbose}) {
+		    vmsg ("Not valid json");
+		}
+		$gs->reply ($fh, {error => 'invalid JSON'});
+		next;
+	    }
+	    if ($gs->{verbose}) {
+		vmsg ("Validated as JSON");
+	    }
+	    my $input = $gs->{jp}->parse ($got);
+	    if (ref $input eq 'HASH') {
+		my $control = $input->{'JSON::Server::control'};
+		if (defined $control) {
+		    if ($control eq 'stop') {
+			if ($gs->{verbose}) {
+			    vmsg ("Received control message to stop");
+			}
+			$gs->reply ($fh, {'JSON::Server::response' => 'stopping'});
+			if ($gs->{verbose}) {
+			    vmsg ("Responded to control message to stop");
+			}
+			$gs->close ($fh);
+			return;
+		    }
+		    if ($control eq 'close') {
+			$gs->reply ($fh, {'JSON::Server::response' => 'closing'});
+			if ($gs->{verbose}) {
+			    vmsg ("Responded to control message to close connection");
+			}
+			$gs->close ($fh);
+			next;
+		    }
+		    warn "Unknown control command '$control'";
+		}
+	    }
+	    $gs->respond ($fh, $input);
 	}
-	$gs->respond ($input);
     }
 }
 
 sub respond
 {
-    my ($gs, $input) = @_;
+    my ($gs, $fh, $input) = @_;
     my $reply;
     if (! $gs->{handler}) {
 	carp "Handler is not set, will echo input back";
@@ -148,21 +176,27 @@ sub respond
     };
     if (! $ok) {
 	carp "Handler crashed: $@";
-	$gs->reply ({error => "Handler crashed: $@"});
+	$gs->reply ($fh, {error => "Handler crashed: $@"});
 	return;
     }
-    $gs->reply ($reply);
+    $gs->reply ($fh, $reply);
 }
 
 sub reply
 {
-    my ($gs, $msg) = @_;
-    my $json_msg = $gs->{jc}->run ($msg);
-    my $sent = $gs->{client}->send ($json_msg);
+    my ($gs, $fh, $msg) = @_;
+    my $json_msg = $gs->{jc}->create ($msg);
+    $json_msg .= chr (0);
+    my $sent = $fh->send ($json_msg);
     if (! defined $sent) {
 	warn "Error sending: $@\n";
     }
-    $gs->{client}->close ();
+}
+
+sub JSON::Server::close
+{
+    my ($gs, $fh) = @_;
+    $fh->close ();
 }
 
 # This is the default callback of the server.
